#include "USchEditor.h"

USchEditor::USchEditor()
{
	CtrlLayout(*this, "  Sch file Editor : ");
	Acceptor(exit, IDEXIT);
	exit.Exit();
	
	selected_table = NULL;
	
	tables.AddRowNumColumn("");
	tables.AddColumn("Name").Edit(editTableName);
//	tables.AddIndex("cols");
	tables.ColumnWidths("1 5");
	tables.AppendLine().Appending().AutoAppending().Removing();
	
	tables.EvenRowColor();
	
	tables.WhenSel = THISBACK(TablesSel);
	tables.WhenUpdateRow = THISBACK(TablesUpdateRow);
	tables.WhenAcceptRow = THISBACK(TablesAcceptRow);
//	tables <<= THISBACK(LoadDropDownRef);
	
	columns.AddRowNumColumn("");
	columns.AddColumn("Name").Edit(editColumnName);
	columns.AddColumn("Type").Edit(editType);
	columns.AddColumn("Length").Edit(editLength);

	pk.AddColumn(columns, "P.K");
	ai.AddColumn(columns, "A.I");
	nn.AddColumn(columns, "N.N");

	columns.AddColumn("Reference").Edit(editRef);
	
	columns.AppendLine().Appending().AutoAppending().Removing();
	columns.EvenRowColor();
	columns.ColumnWidths("1 6 2 2 1 1 1 4");
	
	editLength.Min(0);
		
	editType.Add("INT");
	editType.Add("STRING");
	editType.Add("BOOL");
	editType.Add("DOUBLE");
	editType.Add("DATE");
	editType.Add("TIME");
	editType.Add("serial");
	
	editTableName.SetFilter(CharFilterAlphaToUpper);
	editColumnName.SetFilter(CharFilterAlphaToUpper);
	
	gen_ids = true;

	fromClip <<= THISBACK(FromClip);
	fromFile <<= THISBACK(FromFile);
	toClip <<= THISBACK(ToClip);
	toFile <<= THISBACK(ToFile);
		
	columns.WhenAcceptRow = THISBACK(ColumnsAcceptRow);
	columns.WhenStartEdit = THISBACK(ColumnsStartEdit);
	columns.WhenSel = THISBACK(ColumnsSel);
	columns <<= THISBACK(UpdateObjects);

	columns.WhenUpdateRow = THISBACK(ColumnsUpdateRow);
}

bool USchEditor::Accept()
{
	return PromptYesNo(" are you sur?");
}

void USchEditor::TablesUpdateRow()
{
	if (!tables.IsCursor()) return;
	if (tables.GetCount() <= 0) return;
	
	int i = tables.GetCursor();

	String s = tables.Get(i, 0);

	if(s.IsEqual(selected_table->name)) return;

	//rename table, because it is renamed
	
	Table& t = db.AddTable(s);
	
	t = *selected_table;
	
	db.tables.RemoveKey(selected_table->name);
	
	t.name = s;
	
	selected_table = &t;
	
	columns.SetFocus();
	
	LLOG("TablesUpdateRow");
}

void USchEditor::UpdateView()
{
	columns.Clear();
	tables.Clear();
	
	for(int i = 0; i < db.tables.GetCount(); i++)
	{
		tables.Add(db.tables[i].name);
	}
	
	if(tables.GetCount() > 0)
	{
		selected_table = &db.tables[0];
	}
	else
	{
		selected_table = NULL;
	}
	
	LoadDropDownRef();
}

void USchEditor::LoadDropDownRef()
{
	editRef.Clear();
	
	for(int i = 0; i < db.tables.GetCount(); i++)
	{
		editRef.Add(db.tables.GetKey(i));	
	}
}


bool USchEditor::TablesAcceptRow()
{
	if (!tables.IsCursor()) return false;
	if (tables.GetCount() <= 0) return false;
	
	int i = tables.GetCursor();

	String s = tables.Get(i, 0);

	LLOG("TablesAcceptRow");
	LLOG(s);
	LLOG(selected_table->name);
	
	if (s.IsEqual(selected_table->name)) return true;
	
	
	if (db.tables.Find(s) >= 0)
	{
		Exclamation("Table name duplicated!");
		return false;
	}
	
	LoadDropDownRef();	
	
	return true;
}


void USchEditor::UpdateObjects()
{
	if (!selected_table) return;
		
	selected_table->columns.Clear();
	
	for(int i = 0; i < columns.GetCount(); i++)
	{
		String s = columns.Get(i, 0);
		Column& c = selected_table->columns.Add(s);
		c.name = s;
		s = columns.Get(i, 1);
		c.type = s;
		c.length = columns.Get(i, 2);
		c.in_primarykey = columns.Get(i, 3);
		c.auto_increment = columns.Get(i, 4);
		c.not_null = columns.Get(i, 5);
		c.referance = columns.Get(i, 6);		
	}
}

void USchEditor::ColumnsUpdateRow()
{
	UpdateObjects();
}

void USchEditor::ColumnsStartEdit()
{
	if (!columns.IsCursor()) return ;
	if (columns.GetCount() <= 0) return ;
	
	if(tables.IsEditing())
	{
		if(tables.Accept())
		LoadDropDownRef();
	}
	
	int i = columns.GetCursor();
	String s = columns.Get(i, 1);
	
	if (s.IsEmpty())
	{
		columns.Set(i, 1, "STRING");
		columns.Set(i, 2, 50);
	}
}


void USchEditor::ColumnsSel()
{
	LLOG("ColumnsSel");
//	UpdateObjects();
}

bool USchEditor::ColumnsAcceptRow()
{
	if (!tables.IsCursor()) return false;
	if (tables.GetCount() <= 0) return false;
	if (!columns.IsCursor()) return false;
	if (columns.GetCount() <= 0) return false;
	
	int i = columns.GetCursor();

	String s = columns.Get(i, 1);
	
	int cu = tables.GetCursor();
	
//	Table& t = 	db.tables.Get(tables.Get(cu, 0));
	
	if(columns.Get(i, 1) = "")
	{
		Exclamation("the name is mondatory");
		
		return false;
		
	}

	if(columns.Get(i, 2) = "")
	{
		Exclamation("the type is mondatory");
		
		return false;
		
	}
	
	return true;
}

void USchEditor::TablesSel()
{
	if (!tables.IsCursor()) return;
	if (tables.GetCount() <= 0) return;
	
	String t = tables.Get(0);
	
	// add table if added
	if (db.tables.Find(t) < 0)
	{
		db.tables.Add(t);
		LoadDropDownRef();
	}
	
	// remove table if removed
	for(int i = 0; i < db.tables.GetCount(); i++)
	{
		bool found = false;
		String n = db.tables.GetKey(i);
		for(int j = 0; j < tables.GetCount(); j++)
		{
			String s = tables.Get(j, 0);
			if(n.IsEqual(s))
			{
				found = true;
				break;
			}
		}
		
		if (!found)
		{
			db.tables.RemoveKey(n);
			LoadDropDownRef();
		}
	}
	
	// set new selected table
	selected_table = &db.tables.Get(t);
	
	// reload columns
	columns.Clear();
	
	for(int i = 0; i < selected_table->columns.GetCount(); i++)
	{
		Column& c = db.tables.Get(t).columns[i];
		columns.Add( c.name, c.type,c.length, c.in_primarykey, c.auto_increment, c.not_null, c.referance);
	}
}



GUI_APP_MAIN
{
	USchEditor().Run();
}
