
#include <Core/Core.h>

#include "model.h"

using namespace Upp;
 

Table& ReadTable(CParser& p, Database & db, Index<String>& IdsToGenerate);

void LoadSchFromString(String& str, Database& db)
{
	
	Index<String> IdsToGenerate;
	
	CParser p(str);
	
	db.genId = false;
	
	
	while (!p.IsEof())
	{
		if (p.Id("TABLE"))
		{
			Table& t = ReadTable (p, db, IdsToGenerate);
			
			t.genId = (IdsToGenerate.Find(t.name) >= 0);
		}
		else
		if (p.Id("TABLE_"))
		{
			Table& t = ReadTable (p, db, IdsToGenerate);
			
			t.genId = true;
			IdsToGenerate.Add(t.name);
			
			db.genId = true;
		}
		else
		{
			p.GetChar();
		}				
	}
}

String SaveSchToString(Database& db, bool gen_ids)
{
	StringBuffer sb;
	
	Index<String> generatedid;
	
	String genid_str = "";
	if (gen_ids)
	{
		genid_str = "_";
	}

	for(int i = 0; i < db.tables.GetCount(); i++)
	{
		Table& t = db.tables[i];
		
		sb << "TABLE";
		
		if (gen_ids)
		{
			if(generatedid.Find(t.name)<0)
			{
				sb << genid_str;
				generatedid.Add(t.name);
			}
		}
		
		sb << " (" << t.name << ")\n" ;
		
		int n_keys = 0;
		for(int j = 0; j < t.columns.GetCount() ; j++)
		{
			if(t.columns[j].in_primarykey) n_keys++;
		}
		
		for(int j = 0; j < t.columns.GetCount() ; j++)
		{
			Column& c = t.columns[j];
			
			sb << "\t" << c.type;
			
			if (gen_ids)
			{
				if(generatedid.Find(c.name)<0)
				{
					sb << genid_str ;
					generatedid.Add(c.name);
				}
			}
			
			sb << " (" << c.name; if(c.length > 0) sb << ", " << c.length ;sb << ") " ;
 			
			if(c.in_primarykey && n_keys == 1) sb << " PRIMARY_KEY ";

			if(c.auto_increment) sb << " AUTO_INCREMENT ";
			if(c.not_null) sb << " NOT_NULL ";
			if(c.referance != "") sb << " REFERENCES ( " << c.referance << ")";
			
			sb << "\n";
		}
		
		if(n_keys > 1)
		{
			sb << " PRIMARY_KEYS(\"";
			String v = "";
			for(int j = 0; j < t.columns.GetCount() ; j++)
			{
				if(t.columns[j].in_primarykey) 
				{
					sb << v << t.columns[j].name;
					v = ", "; 
				}
			}
			
			sb << " \")\n";
		}
		
		sb.Cat("END_TABLE\n\n");
		
	}
	
	String s = sb;
	
	return s;
}

void ReadTableElement(String& str, Table& t, Index<String>& IdsToGenerate)
{
	CParser p(str);
	
	if (p.Id("PRIMARY_KEYS"))
	{
		p.Char('(');
		p.Char('"');
		
		while(!p.Char('"') && !p.IsEof())
		{
			String ky = p.ReadId();
			t.columns.Get(ky).in_primarykey = true;
			p.Char(',');
		}

		p.Char(')');
		
		return;
	}
	
	if (p.Id("DUAL_PRIMARY_KEY"))
	{
		p.Char('(');
		
		p.Char('"');

		String ky = p.ReadId();
		t.columns.Get(ky).in_primarykey = true;
		
		p.Char('"');
		p.Char(',');
		p.Char('"');

		ky = p.ReadId();
		t.columns.Get(ky).in_primarykey = true;
		
		p.Char('"');
		p.Char(')');
		
		return;
	}
	
	if (p.IsEof()) return;
	
	String type = p.ReadId();
	
	p.Char('(');
	
	String name = p.ReadId();
	int len = -1;
	if (p.Char(',')) len = p.ReadInt();
	p.Char(')');
	
	Column& c = t.AddColumn(name);
	
	if (len != -1) c.length = len;

	if (type.EndsWith("_"))
	{
		c.genId = true;
		IdsToGenerate.Add(name);
		
		type.Trim(type.GetCount() - 1);
		
		if(t.pdb) t.pdb->genId = true;
	}
	else
	{
		c.genId = (IdsToGenerate.Find(c.name) >= 0);
	}
	
	c.type = type;
	
	while (!p.IsEof())
	{
		if (p.Id("PRIMARY_KEY"))
		{
			c.in_primarykey = true;
		}
		else
		if (p.Id("AUTO_INCREMENT"))
		{
			c.auto_increment = true;
		}
		else
		if (p.Id("NOT_NULL"))
		{
			c.not_null = true;
		}
		else
		if (p.Id("SQLDEFAULT"))
		{
			p.Char('(');
			c.defaultvalue = p.ReadInt();
			p.Char(')');			
		}
		else
		if (p.Id("REFERENCES"))
		{
			p.Char('(');
			c.referance = p.ReadId();
			p.Char(')');			
		}
		else
		{
			p.GetChar();
		}
	}
}

Table& ReadTable(CParser& p, Database & db, Index<String>& IdsToGenerate)
{
	p.Char('(');
	String name = p.ReadId();
	p.Char(')');
	
	Table& t = db.AddTable(name);
	t.name = name;

	StringBuffer sb;
	
	while (!p.IsEof() && !p.Id("END_TABLE"))
	{
		sb << p.GetChar();
	}
	String s = sb;
	Vector<String> ss = Split(s, "\n", true);
	
	for(int i = 0; i < ss.GetCount(); i++)
	{
		String s = TrimBoth(ss[i]);
		if (!s.IsEmpty())
		{
			ReadTableElement(s, t, IdsToGenerate);
		}
	}
	
	return t;
}
